package BPpath.modabrowser;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

 
/*
 * ChooseBlog.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * author: R. Earle Harris (gnudroider@tuxfamily.org) copyrighted 2014
 *
 */

public class ChooseBlog extends ListActivity {

	private final int BLOG_POST = 0;
	private final int ADDEDIT_BLOG = 1;

	private final String[] blogs = {
		"Tiny Bash CGIs\thttp://tinybashcgis.tuxfamily.org/prablog.cgi?fname=$SECONDS&title=$TITLE&content=$CONTENT"
	};

	private PRAFileHandler fhandler = new PRAFileHandler(this);
	private String bloglist = "bloglist";
	private Map<String, String> dbs = new HashMap<String, String>();
	private String sep = "\t"; // must be invalid URL char
	private String link = "";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Intent inc = getIntent();
		link = inc.getStringExtra("LINK");
		setContentView(R.layout.activity_chooseblog);
	}


	@Override
	public void onResume() {
		super.onResume();

		final String[] list = buildList();
		setListAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, list));

		EdgeEffectListView listView = (EdgeEffectListView) getListView();
		listView.setTextFilterEnabled(true);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String name = ((TextView) view).getText().toString();
				if (!name.matches("")){
					String url = dbs.get(name);
					post(name, url);
				}
			}
		});

		final String[] menu_array = { 
			"Add or Edit", 
			"Remove", 
		};

		final Context c = this;

		listView.setOnItemLongClickListener(new OnItemLongClickListener() {
		 
		    @Override 
			public boolean onItemLongClick(AdapterView parentView, View view, int position, long id) {
				final String name = ((TextView) view).getText().toString();
				final String url = dbs.get(name); // full path name in HashMap dbs

				LayoutInflater inflater = getLayoutInflater();
				final ArrayList<String> list = new ArrayList<String>();
				for (int i = 0; i < menu_array.length; ++i) {
					list.add(menu_array[i]);
				}
				
				AlertDialog.Builder builder = new AlertDialog.Builder(c);
				final ArrayAdapter adapter = new ArrayAdapter(c, R.layout.pra_list_item, list);
				builder.setView(inflater.inflate(R.layout.list_context_menu, null))
					.setAdapter(adapter, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						String result = (String) adapter.getItem(which);
						if (result.matches("Add or Edit")){
							Intent intent = new Intent(c, AddEditBlog.class);
							intent.putExtra("NAME",name);
							intent.putExtra("URL", url);
							startActivityForResult(intent, ADDEDIT_BLOG);
							onResume();
						} else if (result.matches("Remove")){
							fhandler.removeLineByToken0(bloglist, name, sep);
							onResume();
						}
						
					}
						})
					.create()
					.show();
				return true;
			}
			});
	}

	private void post(String name, String url){
		Intent i = new Intent(this, BlogPost.class);
		i.putExtra("NAME", name);
		i.putExtra("URL", url);
		i.putExtra("LINK",link);
		startActivityForResult(i, BLOG_POST);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == BLOG_POST) {
			Intent intent = new Intent();
			if (resultCode == RESULT_OK) {
				final String url = data.getStringExtra("URL");
				intent.putExtra("URL", url);
				setResult(RESULT_OK, intent); 
				finish();
			} else {
				;//change blog?
			}
		}else if (requestCode == ADDEDIT_BLOG) {
			if (resultCode == RESULT_OK) {
				String blog = data.getStringExtra("BLOG");
				fhandler.writeUniqLineByTokens(bloglist, blog, sep);
				onResume();
			} else {
				;//??
			}
		}
    }

	private String[] buildList() {
		String[] blogfile = fhandler.getFileAsArray(bloglist);
		List<String> alist = new ArrayList<String>();

		for (String s : blogfile) {
			String[] t = s.split(sep);
			dbs.put(t[0], t[1]);
			alist.add(t[0]);
		}

		for (String s : blogs) {
			String[] t = s.split(sep);
			dbs.put(t[0], t[1]);
			alist.add(t[0]);
		}

		dbs.put("","");
		alist.add("");
		String[] list = new String[alist.size()];
		alist.toArray(list);
		return list;
	}


	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		setResult(RESULT_CANCELED, intent);
		finish();
	}

}
