package BPpath.modabrowser;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.text.InputType;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.String;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

 
/*
 * ChooseContext.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * author: R. Earle Harris (gnudroider@tuxfamily.org) copyrighted 2014
 *
 */
/**
 * ChooseContext.java created and copyrighted: 10 Jun 13
 *
 * author: R. Earle Harris (richard.earle.harris@gmail.com)
 *
 * The author releases this file under the GNU GPL license. See GNU's
 * COPYING file for specifics.
 **/

public class ChooseContext extends ListActivity {

	public Map<String, String> dbs = new HashMap<String, String>();
	public URLDecoder decoder = new URLDecoder();
	public PRAFileHandler fhandler = new PRAFileHandler(this);
	public String sep = "\t"; // must be invalid URL char

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_choosecontext);
	}


	@Override
	public void onResume() {
		super.onResume();

		dbs = new HashMap<String, String>();
		final String[] list = buildList();
		setListAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, list));

		EdgeEffectListView listView = (EdgeEffectListView) getListView();
		listView.setTextFilterEnabled(true);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String result = ((TextView) view).getText().toString();
				if (! result.matches("")){
					String fname = dbs.get(result);
					Intent intent = new Intent();
					intent.putExtra("CHOOSE_CONTEXT", fname);
					setResult(RESULT_OK, intent);
					finish();
				}
			}
		});

		final String[] menu_array = { 
			"New", 
			"Export", 
			"Import", 
			"Remove", 
		};

		final Context c = this;

		listView.setOnItemLongClickListener(new OnItemLongClickListener() {
		 
		    @Override 
			public boolean onItemLongClick(AdapterView parentView, View view, int position, long id) {
				final String name = ((TextView) view).getText().toString();

				LayoutInflater inflater = getLayoutInflater();
				final ArrayList<String> list = new ArrayList<String>();
				for (int i = 0; i < menu_array.length; ++i) {
					list.add(menu_array[i]);
				}
				
				AlertDialog.Builder builder = new AlertDialog.Builder(c);
				final ArrayAdapter adapter = new ArrayAdapter(c, R.layout.pra_list_item, list);
				builder.setView(inflater.inflate(R.layout.list_context_menu, null))
					.setAdapter(adapter, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						String result = (String) adapter.getItem(which);
						if (result.matches("New")){
							newContext();
							onResume();
						} else if (result.matches("Export")){
							exportContext(name);
						} else if (result.matches("Import")){
							importContext();
						} else if (result.matches("Remove")){
							if (! name.matches("MoDa")){
								removeContext(name);
								onResume();
							}
						}
						
					}
						})
					.create()
					.show();
				return true;
			}
			});
	}


	private String[] buildList() {
		String fname = "";
		String title = "";
		String[] tokens;
		File internal = getFilesDir();
		File[] files = internal.listFiles();
		List<String> alist = new ArrayList<String>();
		for (File inFile : files) {
		    fname = inFile.toString();
			tokens	= fname.split("/");
			fname = tokens[tokens.length - 1];
			if (fname.startsWith("context")){
				title = decoder.decode(fname.replace("context", ""));
				alist.add(title);
				dbs.put(title, fname);
			}
		}

		String[] list = new String[alist.size()];
		alist.toArray(list);
		return list;
	}


	private void importContext(){
		File sdcard = Environment.getExternalStorageDirectory();
		File[] files = sdcard.listFiles();
		String[] tokens;
		String src = "";
		String fname = "";

		for (File f : files ){
			src = f.toString();
			tokens	= src.split("/");
			fname = tokens[tokens.length - 1];
			if (fname.startsWith("context")){
				if (dbs.containsValue(fname)){
					getName(fname, src);
				} else {
					finishImport(fname, src);
				}
			}
		}
	}

	private void getName(String fname, String path){
		String Dname = decoder.decode(fname.replace("context", ""));
		final String dir = path;
		final String orig = fname;
		final Dialog dialog = new Dialog(this);
		dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
		dialog.setContentView(R.layout.alert_two_edit);
		TextView t = (TextView) dialog.findViewById(R.id.dTitle);
		t.setText("Duplicate Context Name");
		TextView m = (TextView) dialog.findViewById(R.id.dMessage);
		m.setText("Rename import or overwrite existing context.");
		final EditText e = (EditText) dialog.findViewById(R.id.dEdit);
		e.setInputType(InputType.TYPE_CLASS_TEXT);
		e.setText(Dname);

		Button pos = (Button) dialog.findViewById(R.id.dPos);
		pos.setText("Enter");
		pos.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				String name = e.getText().toString();
				try {
					name = URLEncoder.encode(name,"UTF-8");
				} catch (UnsupportedEncodingException u) { ; }
				finishImport("context" + name, dir);
				dialog.dismiss();
			}
			});
		
		Button neg = (Button) dialog.findViewById(R.id.dNeg);
		neg.setText("Overwrite");
		neg.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				finishImport(orig, dir);
				dialog.dismiss();
			}
			});
		
		dialog.show();
	}

	// fname is new encoded context name, path is original file to import
	private void finishImport(String fname, String path){
		File sdcard = Environment.getExternalStorageDirectory();
		String[] tokens	= path.split("/");
		String src = tokens[tokens.length - 1];
		String[] orig = fhandler.getExtFileAsArray(sdcard.toString()+"/", src);
		String simpleName = fname.replace("context","");
		boolean isNode = true;
		String annot = "";
		String[] t = null;
		for (String s : orig){
			if (isNode){
				fhandler.writeUniqLineByTokens(fname, s, sep);
				t = s.split(sep);
				annot = Integer.toString(t[0].hashCode());
				annot = annot.replace('-', 'm');
				annot = simpleName + annot;
				isNode = false;
			} else if (s.matches("")){
				isNode = true;
			} else {
				FileOutputStream f;
				try {
					boolean append = true;
					f = openFileOutput(annot, Context.MODE_APPEND);
					f.write(s.getBytes());
					f.write(System.getProperty("line.separator").getBytes());
					f.flush();
					f.close();
				} catch (Exception e){ ; }
			}
		}
		onResume();
	}

	private void newContext(){
		final Dialog dialog = new Dialog(this);
		dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
		dialog.setContentView(R.layout.alert_two_edit);
		TextView t = (TextView) dialog.findViewById(R.id.dTitle);
		t.setText("New Context");
		TextView m = (TextView) dialog.findViewById(R.id.dMessage);
		m.setText("Enter Context Name");
		final EditText e = (EditText) dialog.findViewById(R.id.dEdit);
		e.setInputType(InputType.TYPE_CLASS_TEXT);

		Button pos = (Button) dialog.findViewById(R.id.dPos);
		pos.setText("Create");
		pos.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				String name = e.getText().toString();
				String test = dbs.get(name);
				if ((test == null) || (test.matches(""))){
					String fname = "context";
					try {
						fname = fname + URLEncoder.encode(name,"UTF-8");
					} catch (UnsupportedEncodingException u) { ; }
					fhandler.touchFile(fname);
					dialog.dismiss();
					onResume();
				} else {
					dialog.dismiss();
				}
			}
			});
		
		Button neg = (Button) dialog.findViewById(R.id.dNeg);
		neg.setText("Cancel");
		neg.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				dialog.dismiss();
				onResume();
			}
			});
		
		dialog.show();
	}

	private void exportContext(String name){
		String fname = dbs.get(name);
		String prefix = fname.replace("context","");
		File sdcard = Environment.getExternalStorageDirectory();
		String[] file2export = fhandler.getFileAsArray(fname);
		File exportfile = new File(sdcard, fname);
		FileOutputStream writer = null;
		try {
			writer = new FileOutputStream(exportfile);
		} catch (java.io.FileNotFoundException fnf) {;}

		for (String s : file2export) {
			try {
				// write node
				writer.write(s.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();

				// write annotation
				String[] t = s.split(sep);
				String annot = Integer.toString(t[0].hashCode());
				annot = annot.replace('-', 'm');
				annot = prefix + annot;
				String[] orig = fhandler.getFileAsArray(annot);
				for (String st : orig) {
					writer.write(st.getBytes());
					writer.write(System.getProperty("line.separator").getBytes());
					writer.flush();
				}
				
				// write blank line
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		try {
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		toaster(getResources().getString(R.string.context)+" "+
				name+" "+getResources().getString(R.string.exported), Toast.LENGTH_SHORT);
	}


	private void removeContext(String name){
		final String fname = dbs.get(name);
		final Dialog dialog = new Dialog(this);
		dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
		dialog.setContentView(R.layout.alert_two);
		TextView t = (TextView) dialog.findViewById(R.id.dTitle);
		t.setText("Context Deletion");
		TextView m = (TextView) dialog.findViewById(R.id.dMessage);
		m.setText("Remove "+name+"?");

		Button pos = (Button) dialog.findViewById(R.id.dPos);
		pos.setText("Delete");
		pos.setOnClickListener(new OnClickListener() {
				@Override
					public void onClick(View v) {
					String[] t = null;
					String annot = "";
					String[] orig = fhandler.getFileAsArray(fname);
					String prefix = fname.replace("context","");
					for (String s : orig){
						t = s.split(sep);
						annot = Integer.toString(t[0].hashCode());
						annot = annot.replace('-', 'm');
						annot = prefix + annot;
						fhandler.remove(annot);
					}
                    fhandler.remove(fname);
					dialog.dismiss();
					onResume();
				}
			});

		Button neg = (Button) dialog.findViewById(R.id.dNeg);
		neg.setText("Cancel");
		neg.setOnClickListener(new OnClickListener() {
				@Override
					public void onClick(View v) {
					dialog.dismiss();
				}
			});

		dialog.show();

	}


	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		setResult(RESULT_CANCELED, intent);
		finish();
	}


	public void toaster(String text, int length) {
		Toast t = Toast.makeText(this, text, length);
		t.setGravity(Gravity.CENTER, 0, 0);
		t.show();
	}

}
