package BPpath.modabrowser;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.InputType;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import java.io.File;
import java.lang.String;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

 
/*
 * OpenContext.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * author: R. Earle Harris (gnudroider@tuxfamily.org) copyrighted 2014
 *
 */

public class OpenContext extends ListActivity {

	public Map<String, String> bookmarks = new HashMap<String, String>();
	public Map<String, String> ctxts = new HashMap<String, String>();
	public PRAFileHandler fhandler = new PRAFileHandler(this);
	public String contextName = "MoDa";
	public String currentContext = "contextMoDa";
	public String sep = "\t"; // must be invalid URL char
	public String[] Bookmarks;
	public TextView title;
	public URLDecoder decoder = new URLDecoder();
	 
	public final static String firstItem = "Manage Contexts";
	public final static int CHOOSE_CONTEXT = 0;
	
	public boolean contextTravel = false;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Intent i = getIntent();
		contextTravel = i.getBooleanExtra("TRAVEL", false);
		String newContext = i.getStringExtra("CONTEXT");
		if ((newContext != null) && (!newContext.matches(""))){
			currentContext = newContext;
			contextName = decoder.decode(currentContext.replace("context", ""));
		}
		setContentView(R.layout.activity_open_context);
		title = (TextView) findViewById(R.id.title);
	}

	@Override
	public void onResume() {
		super.onResume();
		setBookmarks();
		setListAdapter(new ArrayAdapter<String>(this, R.layout.node, Bookmarks));
		EdgeEffectListView listView = (EdgeEffectListView) getListView();
		listView.setTextFilterEnabled(true);

		final Context c = this;
		listView.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String url;
				String title = (String) ((TextView) view).getText();
				if (title.matches(firstItem)) {
					url = firstItem;
					Intent intent = new Intent(c, ChooseContext.class);
					startActivityForResult(intent, CHOOSE_CONTEXT);
				} else if (title.matches("")) {
					; // cosmetic
				} else {
					url = bookmarks.get(title);
					Intent intent = new Intent();
					intent.putExtra("URL", url);
					intent.putExtra("CONTEXT", currentContext);
					setResult(RESULT_OK, intent);
					finish();
				}
			}
		});

		final String[] menu_array = { 
			"Annotate", 
			"Remove", 
			"Rename", 
			"Reorder", 
			"Share", 
			"Move", 
		};

		listView.setOnItemLongClickListener(new OnItemLongClickListener() {
		 
		    @Override 
			public boolean onItemLongClick(AdapterView parentView, View view, int position, long id) {
				final String node = ((TextView) view).getText().toString();
				final String url = bookmarks.get(node); // full path name in HashMap dbs
				final String ctxt = currentContext.replace("context","");

				LayoutInflater inflater = getLayoutInflater();
				final ArrayList<String> list = new ArrayList<String>();
				for (int i = 0; i < menu_array.length; ++i) {
					list.add(menu_array[i]);
				}
				
				AlertDialog.Builder builder = new AlertDialog.Builder(c);
				final ArrayAdapter adapter = new ArrayAdapter(c, R.layout.pra_list_item, list);
				builder.setView(inflater.inflate(R.layout.list_context_menu, null))
					.setAdapter(adapter, new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						String result = (String) adapter.getItem(which);
						if (result.matches("Annotate")){
							Intent intent = new Intent(c, Annotate.class);
							intent.putExtra("NODE", node);
							intent.putExtra("CONTEXT", currentContext);
							startActivity(intent);
						} else if (result.matches("Remove")){
							final Dialog dialog1 = new Dialog(c);
							dialog1.requestWindowFeature(Window.FEATURE_NO_TITLE);
							dialog1.setContentView(R.layout.alert_two);
							TextView t = (TextView) dialog1.findViewById(R.id.dTitle);
							t.setText("Remove Node");
							TextView m = (TextView) dialog1.findViewById(R.id.dMessage);
							m.setText(node);

							Button pos = (Button) dialog1.findViewById(R.id.dPos);
							pos.setText("Remove");
							pos.setOnClickListener(new OnClickListener() {
								@Override
								public void onClick(View v) {
									fhandler.removeLineByToken0(currentContext, node, sep);
									String fname = Integer.toString(node.hashCode());
									fname = fname.replace('-', 'm');
									fhandler.remove(fname);
									dialog1.dismiss();
									onResume();
								}
								});
							
							Button neg = (Button) dialog1.findViewById(R.id.dNeg);
							neg.setText("Cancel");
							neg.setOnClickListener(new OnClickListener() {
								@Override
								public void onClick(View v) {
									dialog1.dismiss();
								}
								});
							
							dialog1.show();
						} else if (result.matches("Rename")){
							final Dialog dialog2 = new Dialog(c);
							dialog2.requestWindowFeature(Window.FEATURE_NO_TITLE);
							dialog2.setContentView(R.layout.alert_two_edit);
							TextView t = (TextView) dialog2.findViewById(R.id.dTitle);
							t.setText("Rename Node");
							TextView m = (TextView) dialog2.findViewById(R.id.dMessage);
							m.setText("Enter new name.\nCancel aborts.");
							final EditText e = (EditText) dialog2.findViewById(R.id.dEdit);
							e.setInputType(InputType.TYPE_CLASS_TEXT);
							e.setText(node);
							Button pos = (Button) dialog2.findViewById(R.id.dPos);
							pos.setText("Set Name");
							pos.setOnClickListener(new OnClickListener() {
								@Override
								public void onClick(View v) {
									String newName = e.getText().toString();
									String line = newName + sep + url;
									fhandler.removeLineByToken0(currentContext, node, sep);
									fhandler.writeUniqLineByTokens(currentContext, line, sep);
									String fname = Integer.toString(node.hashCode());
									fname = fname.replace('-', 'm');
									fname = ctxt + fname;
									String[] orig = fhandler.getFileAsArray(fname);
									String fname2 = Integer.toString(newName.hashCode());
									fname2 = fname2.replace('-', 'm');
									fname2 = ctxt + fname2;
									fhandler.writeArrayToFile(orig, fname2);
									fhandler.remove(fname);
									dialog2.dismiss();
									onResume();
								}
								});
		
							Button neg = (Button) dialog2.findViewById(R.id.dNeg);
							neg.setText("Cancel");
							neg.setOnClickListener(new OnClickListener() {
								@Override
								public void onClick(View v) {
									dialog2.dismiss();
								}
								});
							
							dialog2.show();
						} else if (result.matches("Reorder")){
							Intent i = new Intent(c, ReorderContext.class);
							i.putExtra("CONTEXT", currentContext);
							startActivity(i);
						} else if (result.matches("Share")){
							String fname = Integer.toString(node.hashCode());
							fname = fname.replace('-', 'm');
							fname = ctxt + fname;
							String[] orig = fhandler.getFileAsArray(fname);
							String content = "";
							for (String s : orig) {
								content = content + s + "\n";
							}
							Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
							emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, node);
							emailIntent.putExtra(android.content.Intent.EXTRA_TEXT,
								 "\n\n<a href=\""+url+"\">"+node+"</a>" +  "\n\n" + 
								 content + "\n\n    ::shared from Mo Da Browser::");
							emailIntent.setType("text/plain");
							startActivity(Intent.createChooser(emailIntent, "Email node with ..."));
						} else if (result.matches("Move")){
							String fname = "";
							String title = "";
							String[] tokens;
							File internal = getFilesDir();
							File[] files = internal.listFiles();
							List<String> alist = new ArrayList<String>();
							for (File inFile : files) {
								fname = inFile.toString();
								tokens	= fname.split("/");
								fname = tokens[tokens.length - 1];
								if ((fname.startsWith("context")) && (!fname.matches(currentContext))){
									title = decoder.decode(fname.replace("context", ""));
									alist.add(title);
									ctxts.put(title, fname);
								}
							}

							Collections.sort(alist);
							String[] list = new String[alist.size()];
							alist.toArray(list);
							final String[] listf = list;

							final ArrayAdapter adapter = new ArrayAdapter(c, R.layout.pra_list_item, list);
							AlertDialog.Builder b = new AlertDialog.Builder(c);
							b.setAdapter(adapter, new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog, int which) {
										String dest = ctxts.get(listf[which]);
										String line = node + sep + url;
										fhandler.removeLineByToken0(currentContext, node, sep);
										fhandler.writeUniqLineByTokens(dest, line, sep);
										onResume();
									}
								});
							LayoutInflater inflater = getLayoutInflater();
							View vw = inflater.inflate(R.layout.titlemove, null);
							b.setCustomTitle(vw);
							b.create();
							b.show();
						}
					}
						})
					.create()
					.show();
				return true;
			}
			});

		title.setText(getResources().getString(R.string.context) + ": "+ contextName);
	}


	public void setBookmarks() {
		List<String> tmpBookmarks = new ArrayList<String>();
		String[] file = fhandler.getFileAsArray(currentContext);
		bookmarks.put(firstItem, ""); // leave here to sync array indices
		tmpBookmarks.add(firstItem);
		for (String s : file) {
			String[] t = s.split(sep);
			bookmarks.put(t[0], t[1]);
			tmpBookmarks.add(t[0]);
		}
		Bookmarks = new String[tmpBookmarks.size()];
		tmpBookmarks.toArray(Bookmarks);

	}


	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == CHOOSE_CONTEXT) {
			if (resultCode == RESULT_OK) {
				currentContext = data.getStringExtra("CHOOSE_CONTEXT");
				contextName = decoder.decode(currentContext.replace("context", ""));
				if (contextTravel){
					Intent intent = new Intent();
					intent.putExtra("URL", "none");
					intent.putExtra("CONTEXT", currentContext);
					setResult(RESULT_OK, intent);
					finish();
				}
			} else {
				String[] file = fhandler.getFileAsArray(currentContext);
				if (file.length == 0){
					currentContext = "contextMoDa";
					contextName = "MoDa";
				}
			}
		}
		onResume();
    }


	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		intent.putExtra("URL", "none");
		intent.putExtra("CONTEXT", currentContext);
		setResult(RESULT_OK, intent);
		finish();
	}



    // length gets Toast.LENGTH_LONG|SHORT
	public void toaster(String text, int length) {
		Toast t = Toast.makeText(this, text, length);
		t.setGravity(Gravity.CENTER, 0, 0);
		t.show();
	}


}
