package BPpath.modabrowser;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.Html;
import android.view.Gravity;
import android.view.View;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.Toast;
import java.lang.Runnable;
import java.util.ArrayList;

 
/*
 * ReorderContext.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fork of Apache code by: Eric Harlow (c) 2010 
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2016
 *
 */

public class ReorderContext extends ListActivity {

	public PRAFileHandler fhandler = new PRAFileHandler(this);
	public String context = "";
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
		Intent i = getIntent();
		context = i.getStringExtra("CONTEXT");
		ArrayList<String> nodes = (ArrayList)fhandler.getFileAsList(context);
        setContentView(R.layout.activity_reordercontext);
        setListAdapter(new DragNDropAdapter(this, new int[]{R.layout.dragitem}, new int[]{R.id.TextView01}, nodes));
        ListView listView = getListView();
        if (listView instanceof DragNDropListView) {
        	((DragNDropListView) listView).setDropListener(mDropListener);
        	((DragNDropListView) listView).setRemoveListener(mRemoveListener);
        	((DragNDropListView) listView).setDragListener(mDragListener);
        }
    }

	private DropListener mDropListener = 
		new DropListener() {
        public void onDrop(int from, int to) {
        	ListAdapter adapter = getListAdapter();
        	if (adapter instanceof DragNDropAdapter) {
        		((DragNDropAdapter)adapter).onDrop(from, to);
        		getListView().invalidateViews();
        	}
        }
    };
    
    private RemoveListener mRemoveListener =
        new RemoveListener() {
        public void onRemove(int which) {
        	ListAdapter adapter = getListAdapter();
        	if (adapter instanceof DragNDropAdapter) {
        		((DragNDropAdapter)adapter).onRemove(which);
        		getListView().invalidateViews();
        	}
        }
    };
    
    private DragListener mDragListener =
    	new DragListener() {

    	int backgroundColor = 0xe0103010;
    	int defaultBackgroundColor;
    	
			public void onDrag(int x, int y, ListView listView) {
				// TODO Auto-generated method stub
			}

			public void onStartDrag(View itemView) {
				itemView.setVisibility(View.INVISIBLE);
				defaultBackgroundColor = itemView.getDrawingCacheBackgroundColor();
				itemView.setBackgroundColor(backgroundColor);
				ImageView iv = (ImageView)itemView.findViewById(R.id.ImageView01);
				if (iv != null) iv.setVisibility(View.INVISIBLE);
			}

			public void onStopDrag(View itemView) {
				itemView.setVisibility(View.VISIBLE);
				itemView.setBackgroundColor(defaultBackgroundColor);
				ImageView iv = (ImageView)itemView.findViewById(R.id.ImageView01);
				if (iv != null) iv.setVisibility(View.VISIBLE);
			}
    	
    };

	public void done(View view){
        ListView listView = getListView();
		ListAdapter adapter = listView.getAdapter();
		int count = adapter.getCount();
		ArrayList<String> list = new ArrayList<String>();
		for(int i=0; i<count; i++){
			list.add((String)adapter.getItem(i));
		}
		fhandler.writeListToFile(list, context);
		finish();
	}
    

	private boolean doubleBackPressedOnce = false;

	@Override
	public void onBackPressed() {
		if (doubleBackPressedOnce) {
			super.onBackPressed();
			return;
		} else {
			Toast.makeText(this, R.string.exit_bkbtn, Toast.LENGTH_SHORT).show();
		}
		doubleBackPressedOnce = true;
		new Handler().postDelayed(new Runnable() {

			@Override
			public void run() {
				doubleBackPressedOnce = false;
			}
		}, 2000);
	}


    // length gets Toast.LENGTH_LONG|SHORT
	public void toaster(String text, int length) {
		Toast t = Toast.makeText(this, Html.fromHtml("<h1>"+text+"</h1>"), length);
		t.setGravity(Gravity.CENTER, 0, 0);
		t.show();
	}


}
